from rest_framework.response import Response
from rest_framework import status
from drf_api_lens.validators import RequiredParameterValidator
from drf_api_lens.utils import ErrorAnalysis

def capture_error(validate_api_parameters=[]):
    """
    This decorator capture errors and validates API parameters before allowing access to the decorated view function.
    It can be used by adding @capture_error(validate_api_parameters=["param1", "param2"]) above a view function.
    The decorator takes an optional argument, validate_api_parameters, which is a list of required API parameters.
    """

    def decorator(view_func):
        def wrapper(self, request, *args, **kwargs):
            try:
                if validate_api_parameters:
                    validator = RequiredParameterValidator(
                        request, validate_api_parameters
                    )
                    is_valid, missing_parameters = validator.validate()
                    if not is_valid:
                        return Response(
                            data={
                                "error": True,
                                "data": [],
                                "message": f"Missing parameters: {', '.join(missing_parameters)}",
                            },
                            status=status.HTTP_406_NOT_ACCEPTABLE,
                        )
                return view_func(self, request, *args, **kwargs)

            except Exception as exception:
                error_message = ErrorAnalysis.detect_error(exception)
                return Response(
                    status=status.HTTP_406_NOT_ACCEPTABLE,
                    data={
                        "error": True,
                        "data": [],
                        "message": error_message,
                    },
                )

        return wrapper

    return decorator



# Generated by Django 4.1.7 on 2023-03-11 07:59

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ErrorCapture',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('user', models.CharField(blank=True, max_length=256, null=True)),
                ('url_requested', models.CharField(blank=True, max_length=1024, null=True)),
                ('request_headers', models.TextField(blank=True, null=True)),
                ('request_body', models.TextField(blank=True, null=True)),
                ('query_params', models.TextField(blank=True, null=True)),
                ('request_method', models.CharField(blank=True, max_length=8, null=True)),
                ('ip_address', models.CharField(blank=True, max_length=50, null=True)),
                ('response_body', models.TextField(blank=True, null=True)),
                ('response_status_code', models.PositiveSmallIntegerField(blank=True, null=True)),
                ('execution_time', models.DecimalField(blank=True, decimal_places=5, max_digits=8, null=True)),
            ],
            options={
                'db_table': 'error_capture',
            },
        ),
    ]



import time
from django.conf import settings
from drf_api_lens.store_capture_error import ErrorCaptureOperation
from drf_api_lens.utils import CleaningData


class ErrorCapturingMiddleware:
    """
    This class defines a middleware component in a Python web application that captures errors that occur
    during the processing of requests and stores them in a database if user is allowed. The middleware is
    called for each incoming request and is responsible for handling the request and generating a response.
    """
    def __init__(self, get_response):
        self.get_response = get_response
        self.STORE_CAPTURE_ERROR_TO_DATABASE = getattr(settings, 'STORE_CAPTURE_ERROR_TO_DATABASE', False)

    def __call__(self, request):
        if self.STORE_CAPTURE_ERROR_TO_DATABASE:
            start_time = time.time()
            # code to be executed for each request before the view (and later middleware) are called.
            response = self.get_response(request)
            # code to be executed for each request/response after the view is called.
            """
            200: HTTP_200_OK
            201: HTTP_201_CREATED
            202: HTTP_202_ACCEPTED
            203: HTTP_203_NON_AUTHORITATIVE_INFORMATION
            204: HTTP_204_NO_CONTENT
            205: HTTP_205_RESET_CONTENT
            206: HTTP_206_PARTIAL_CONTENT
            207: HTTP_207_MULTI_STATUS
            208: HTTP_208_ALREADY_REPORTED
            226: HTTP_226_IM_USED
            """
            if response.status_code not in [200,201,202,203,204,205,206,207,208,226]:
                data = CleaningData.cleaning_and_build_data_dict(request,response,start_time)
                ErrorCaptureOperation.push(data=data)
        else:
            response = self.get_response(request)
        return response


from drf_api_lens.models import ErrorCapture

class ErrorCaptureOperation:
        """
        The ErrorCaptureOperation class is a Python class that contains a static method called push(). 
        This class is likely used for capturing and logging errors that occur during runtime in an application.
        """
        @staticmethod
        def push(data):
                ErrorCapture.objects.create(**data)


class RequiredParameterValidator:
    """
    This is the RequiredParameterValidator class, which is used to validate that required
    parameters are present in a Django API request. It includes an init method that sets the
    request and required_parameters attributes, and a validate method that iterates through
    the required_parameters and checks if each one is present in the request data.
    If any required parameters are missing, the validate method returns False and a list of missing
    parameters. If all required parameters are present, it returns True and None.
    """

    def __init__(self, request, required_parameters):
        self.request = request
        self.required_parameters = required_parameters

    def validate(self):
        missing_parameters = []
        for parameter in self.required_parameters:
            if parameter not in self.request.data:
                missing_parameters.append(parameter)

        if len(missing_parameters) > 0:
            return False, missing_parameters
        return True, None



import re
from django.conf import settings
import json
import time

class RequestAnalysis:
    """
    The class RequestAnalysis contains a collection of static methods that can be used
    to analyze a request object in a web application. Each method is responsible for retrieving
    a specific aspect of the request object, such as headers, URL, parameters, data, method, user, 
    and client IP address.
    """
    
    @staticmethod
    def get_headers(request=None):
        """
        This method retrieves the HTTP headers from the request 
        object and returns them in a dictionary format.
        """
        regex = re.compile('^HTTP_')
        return dict((regex.sub('', header), value) for (header, value)
                in request.META.items() if header.startswith('HTTP_'))

    @staticmethod
    def get_request_url(request=None):
        """
        This method retrieves the full URL of the request object.
        """
        return request.build_absolute_uri()

    @staticmethod
    def get_request_params(request=None):
        """
        This method retrieves the query parameters from the 
        request object and returns them in a dictionary format.
        """
        return dict(request.GET)

    @staticmethod
    def get_request_data(request=None):
        """
        This method retrieves the request data from the request object. 
        It first attempts to parse the data as JSON, but if that fails,
        it will fall back to a combination of the FILES and POST dictionaries.
        """
        try:
            request_data = json.loads(request.body) if request.body else ''
        except Exception:
            request_data = {**dict(request.FILES),**dict(request.POST)}

        return request_data

    @staticmethod
    def get_request_method(request=None):
        """
        This method retrieves the HTTP method (e.g., GET, POST, PUT, DELETE)
        from the request object.
        """
        return request.method

    @staticmethod
    def get_requested_user(request=None):
        """
        This method retrieves the user object associated with the request object, if any.
        """
        return request.user

    @staticmethod
    def get_request_client_ip(request):
        """
        This method retrieves the client IP address associated with the
        request object. It first checks for the HTTP_X_FORWARDED_FOR header 
        and uses that if available. Otherwise, it falls back to the REMOTE_ADDR header. 
        If all else fails, it returns an empty string.
        """
        try:
            if x_forwarded_for := request.META.get('HTTP_X_FORWARDED_FOR'):
                remote_ip = x_forwarded_for.split(',')[0]
            else:
                remote_ip = request.META.get('REMOTE_ADDR')
            return remote_ip
        except Exception:
            return ''

class ResponseAnalysis:
    def __init__(self,response) -> None:
        """
        This is the constructor of the class, which takes in a response parameter
        that represents the response received from a web request. The constructor 
        initializes the response attribute of the class with this parameter.
        """
        self.response=response

    def get_response_data(self):
        """
        This method analyzes the response attribute and returns the body of the response
        in a format that is suitable for further processing. It does this by checking the 
        content-type header of the response and using it to determine how to parse the 
        response body. If the content-type is JSON, the method parses the response body 
        using the json.loads() function and returns the resulting JSON object. If the 
        content-type is any other type, the method returns the content-type string itself.
        """
        CONTENT_TYPES = (
            'application/json',    
            'application/xml',
            'application/xhtml+xml',
            'application/pdf',
            'application/zip',
            'application/x-www-form-urlencoded',
            'application/javascript',
            'application/octet-stream',
            'application/vnd.ms-excel',
            'application/vnd.ms-powerpoint',
            'application/vnd.ms-word',
            'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
            'application/vnd.openxmlformats-officedocument.presentationml.presentation',
            'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
            'application/vnd.api+json',
            'audio/mpeg',
            'audio/wav',
            'image/gif',
            'image/jpeg',
            'image/png',
            'image/svg+xml',
            'multipart/form-data',
            'text/html',
            'text/plain',
            'text/xml',
            'text/csv'
        )

        content_type = self.response.get('content-type')
        if content_type in CONTENT_TYPES:
            if content_type != 'application/json':
                response_body = content_type
            else:
                response_body = json.loads(self.response.content).get("message")
        else:
            response_body = []
        return response_body


    def get_response_status_code(self):
        """
        This method returns the HTTP status code of the response. 
        It does this by calling the status_code attribute of the response object.
        """
        return self.response.status_code


class ErrorAnalysis:
    """
    This class seems to be designed to help with error handling and
    debugging by providing useful information about where an error occurred.
    """

    @staticmethod
    def detect_error(exception):
        """
        The detect_error method takes an exception as input, and
        uses the get_error_info method to extract the file path,
        line number, and function name where the error occurred.
        It then formats these values and the exception message 
        into a string, which it returns as the error message.
        """
        error_file_path, error_line_number, function_name = ErrorAnalysis.get_error_info()
        error_message = "An error occurred in file '{}' at line {}: '{}' in function '{}'".format(
            error_file_path, error_line_number,str(exception) ,function_name)
        return error_message
        

    @staticmethod
    def get_error_info() -> None:
        """
        The get_error_info method uses the traceback module to extract 
        information about the current exception, such as the file path,
        line number, and function name. It then returns these values.
        """
        import sys
        import traceback
        tb = traceback.extract_tb(sys.exc_info()[2])
        error_file, error_line, function_name, text = tb[-1]
        return error_file, error_line, function_name


class CleaningData:
    """
    This is a class definition for a Python class called "CleaningData".The 
    class contains a single static method called "cleaning_and_build_data_dict" 
    that takes three parameters: "request", "response", and "start_time".

    Within the method, an instance of a class called "ResponseAnalysis" is created
    using the "response" parameter, which is used to analyze the response and extract
    information such as the response data and status code. The method also uses another 
    class called "RequestAnalysis" to extract information about the request such as the 
    requested user, URL, headers, body, parameters, and method. The method then returns 
    a dictionary that contains all of this information, as well as the execution time of the method.
    """

    @staticmethod
    def cleaning_and_build_data_dict(request,response,start_time):
        response_analysis = ResponseAnalysis(response=response)
        data = dict(
            user=RequestAnalysis.get_requested_user(request=request),
            url_requested=RequestAnalysis.get_request_url(request=request),
            request_headers=RequestAnalysis.get_headers(request=request),
            request_body=RequestAnalysis.get_request_data(request=request),
            query_params=RequestAnalysis.get_request_params(request=request),
            request_method=RequestAnalysis.get_request_method(request=request),
            ip_address=RequestAnalysis.get_request_client_ip(request=request),
            response_body=response_analysis.get_response_data(),
            response_status_code=response_analysis.get_response_status_code(),
            execution_time=(time.time() - start_time)*1000,
        )
        return data


from django.db import models

# Create your models here.
class TimeStampModel(models.Model):
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        abstract = True

class ErrorCapture(TimeStampModel):
        user = models.CharField(max_length=256,null=True,blank=True)
        url_requested = models.CharField(max_length=1024,null=True,blank=True)
        request_headers = models.TextField(null=True,blank=True)
        request_body = models.TextField(null=True,blank=True)
        query_params = models.TextField(null=True,blank=True)
        request_method = models.CharField(max_length=8,null=True,blank=True)
        ip_address = models.CharField(max_length=50,null=True,blank=True)
        response_body = models.TextField(null=True,blank=True)
        response_status_code = models.PositiveSmallIntegerField(null=True,blank=True)
        execution_time = models.DecimalField(decimal_places=5, max_digits=8,null=True,blank=True)

        class Meta:
                db_table = 'error_capture'